package net.sf.corn.gate;

import java.io.Serializable;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;


/**
 * Represents a response of a service execution.
 * A response body can have either a returned value or an exception.
 * 
 * @author Serhat Dirik
 *
 */
public class ServiceResponse implements Serializable{
 
	private static final long serialVersionUID = -1738607621680964905L;
	
	private Object    returnedValue = null;
	private Class<?>  returnedType =null;
	private Throwable returnedException =null;
	private boolean   expectedException=false;
		
	private Map<String,Object> headers = new ConcurrentHashMap<String, Object>();
	
	public ServiceResponse(Object returnValue) {
		super();
		this.returnedValue = returnValue;
		this.returnedType = returnValue == null ? null : returnedValue.getClass();
	}
	
	public ServiceResponse(Object returnValue, Class<?> returnType) {
		super();
		this.returnedValue = returnValue;
		this.returnedType = returnType;
	}
	

	public ServiceResponse(Throwable exception, boolean isExpected) {
		super();
		this.returnedException = exception;
		this.expectedException = isExpected;
	}

	public ServiceResponse(Throwable exception) {
		super();
		this.returnedException = exception;
	}
	

	/* (non-Javadoc)
	 * @see org.gate.call.IServiceResponse#getReturnedValue()
	 */
	public Object getReturnedValue() {
		return returnedValue;
	}

	public Class<?> getReturnedType() {
		return returnedType;
	}

	/* (non-Javadoc)
	 * @see org.gate.call.IServiceResponse#getReturnedException()
	 */
	public Throwable getReturnedException() {
		return returnedException;
	}


	/* (non-Javadoc)
	 * @see org.gate.call.IServiceResponse#isExceptionTypeExpected()
	 */
	public boolean isExceptionTypeExpected() {
		return expectedException;
	}
	
	/* (non-Javadoc)
	 * @see org.gate.call.IServiceResponse#hasError()
	 */
	public boolean hasError(){
		return returnedException != null;
	}
	
	/* (non-Javadoc)
	 * @see org.gate.call.IServiceResponse#hasResult()
	 */
	public boolean hasResult(){
		return returnedException == null;
	}
	/* (non-Javadoc)
	 * @see org.gate.call.IServiceResponse#containsHeaderKey(java.lang.String)
	 */
	public boolean containsHeaderKey(String key) {
		return headers.containsKey(key);
	}

	/* (non-Javadoc)
	 * @see org.gate.call.IServiceResponse#getFromHeader(java.lang.String)
	 */
	public Object getFromHeader(String key) {
		return headers.get(key);
	}

	public Object addToHeader(String key, Object value) {
		return headers.put(key, value);
	}

	public Object removeFromHeader(String key) {
		return headers.remove(key);
	}

	/* (non-Javadoc)
	 * @see org.gate.call.IServiceResponse#headerKeySet()
	 */
	public Set<String> headerKeySet() {
		return headers.keySet();
	}


	@Override
	public String toString() {
		return "ServiceResponse [returnedValue=" + returnedValue
				+ ", returnedType=" + returnedType + ", returnedException="
				+ returnedException +", expectedException="
				+ expectedException + ", headers=" + headers + "]";
	}
	
}
